//****************************************************************************/
/**   vjbox  Utilitie-Routinen fr die Mailbox                             **/
/**                                                                        **/
/** Copyright (C) 1997,1999 Jochen Tillmann-Sarrazin, DG6VJ                **/
/** All rights reserved.                                                   **/
/****************************************************************************/


#include "boxutil.h"
#include "toolbox.h"

#include <string.h>


char months_small[12][4]={"jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"};



/* Eine bestimmte Anzahl Microsekunden warten */
void sleep_usec(int us)
{
  struct timeval tv;		/* fr die zeit beim select() */

  tv.tv_sec = 0;
  tv.tv_usec = us;
  select(0,NULL,NULL,NULL,&tv);
}



/* Guckt, ob das bergebene Rufzeichen (*klein*buchstaben) im badcalls - File
 * ist.
 * Rckgabe: 1, wenn ja - sonst 0
 */
int is_in_badcalls(char *call)
{
  FILE *fd;
  char str[100];
  int i;


  sprintf(str,"%s/system/badcalls.box",boxroot);
  if ((fd = fopen(str,"r")) == NULL) return(0);
  
  while ((fgets(str,99,fd)) != NULL) {
    if ((i = POS('\n',str)) != -1) str[i] = '\0';
    
    if (!strcmp(call,str)) {
      fclose(fd);
      return(1);
    }
  }
  fclose(fd);
  
  return(0);
}


/* SSID vom Call abschneiden */
void CutSSID(char *dest, const char *src)
{
  int i=0;
  while ((src[i] != '-') && (src[i] != '\0')) {
    dest[i] = src[i];
    i++;
  }
  dest[i] = '\0';
}



/* Lockt einen Teil eines Files oder das ganze File (siehe fcntl - Hilfe) */
int lockit(int fd, int type, off_t offset, int whence, off_t len)
{
  struct flock lock;
  int fertig = 0, back;
  int lasterror=0;
  
  
  lock.l_type = type;		/* F_RDLCK oder F_WRLCK oder F_UNLCK*/
  lock.l_start = offset;	/* Byte-Offset bezogen auf l_whence */
  lock.l_whence = whence;	/* SEEK_SET, SEEK_CUR, SEEK_END */
  lock.l_len = len;		/* Byteanzahl, 0 = bis EOF */

  do {
    if (fcntl(fd,F_SETLK,&lock) < 0) {
      /* Ein Fehler ist aufgetreten - offensichtlich ist das File schon
       * anderweitig gelockt worden
       */
      if (errno == EACCES) {
        /* Zugriffsfehler */
        fertig = 1;
        back = -1;
      } else if (errno == EAGAIN) {
               /* Nochmal probieren - file ist gelockt */
               /* Kurze Zeit warten */
               sleep_usec(5000);
             }
             else if (lasterror != errno) {
                    syslog(LOG_ERR,"lockit()-error: %s (%i)\n",strerror(errno),errno); 
/*                    box_error(strerror(errno),errno);*/
                    lasterror = errno;
                  }
    }
    else {
      fertig = 1;
      back = 0;
    }
  }
  while (fertig == 0);
  return(back);
}



/* Wenn das bergebene Call wirklich ein Usercall ist, wird 1 zurckgegeben, sonst 0 */
int rufz(char *call)
{
  char *tmp;
  int i;
  
  tmp = (char *)strdup(call);
  
  i = 0;
  while (call[i] != '\0') {
    if (((call[i] >= 'A') && (call[i] <= 'Z')) || ((call[i] >= 'a') && (call[i] <= 'z')))
       tmp[i] = 'B';
    else
       if ((call[i] >= '0') && (call[i] <= '9'))
          tmp[i] = 'Z';
       else
          tmp[i] = 'X';
    i++;
  }

  if ((!strcmp(tmp,"BBZB")) ||
     (!strcmp(tmp,"BBZBB")) ||
     (!strcmp(tmp,"BBZBBB")) ||
     (!strcmp(tmp,"BZBB")) ||
     (!strcmp(tmp,"BZBBB")) ||
     (!strcmp(tmp,"ZBZBB")) ||
     (!strcmp(tmp,"ZBZBBB")) ||
     (!strcmp(tmp,"BZZBB")) ||
     (!strcmp(tmp,"BZZBBB"))) {
    free(tmp);
    return(1);
  }
  return(0);
}





/* In den String eine neue BID schreiben. Kann nicht lnger als 12 Zeichen lang werden
   (+1 Byte ('\0')) 
   
   Sie hat folgendes Aussehen:
   
   DMYLL_CCCCCC
   
   D = Day (Aktueller Tag)
   M = Month (Aktueller Monat)
   Y = Jahr (Aktuelles Jahr (in Jahren seit 1990): 96 = 6, 2000 = 10)
   L = Laufende Zahl
   C = Call der Mailbox
*/
void NewBID(char *str)
{
  char Alpha[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  time_t zeit;
  struct tm tm_brk;
  int fd,count,i;
  s_BoxSendDataFile tmp;
  char filename[100];
  char temp[256];
  
  
  zeit = time(NULL);
  tm_brk = *gmtime(&zeit);
  
  str[0] = Alpha[tm_brk.tm_mday];
  str[1] = Alpha[tm_brk.tm_mon+1];
  str[2] = Alpha[(tm_brk.tm_year - 90) % 36];
  
  /* Laufende Nummer aus der Datei boxsend.dat holen */
  sprintf(filename,"%s/system/boxsend.dat",boxroot);
  
  fd = open(filename,(O_RDWR));

  if (fd == -1) {
    /* Fehler! Wahrscheinlich existiert das File nicht */
    if (errno == ENOENT) {
      /* Existierte wirklich nicht - anlegen */
      fd = open(filename,(O_RDWR|O_CREAT));
      
      /* Zugriffsrechte einstellen */
      fchmod(fd,S_IRUSR|S_IWUSR);
      
      /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
      lockit(fd,F_WRLCK,0,SEEK_SET,0);
      
      tmp.day = tm_brk.tm_mday;
      tmp.mon = tm_brk.tm_mon+1;
      tmp.year = tm_brk.tm_year;
      tmp.lfd_nr = 0;
      write(fd,&tmp,sizeof(tmp));
    }
    else {
      /* Fehlermeldung ins System-Verzeichnis schreiben */
      str[0] = '\0';
/*      sprintf(temp,"(NewBID:1) %s",strerror(errno));
      box_error(temp,errno);*/
      return;
    }
  }
  else {
    /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
    lockit(fd,F_WRLCK,0,SEEK_SET,0);
  
    count = read(fd,&tmp,sizeof(tmp));
    
    lseek(fd,0,SEEK_SET);
    if (count != sizeof(tmp)) {
      /* Fileinhalt ist entweder beschdigt oder das File ist ganz leer */
      tmp.day = tm_brk.tm_mday;
      tmp.mon = tm_brk.tm_mon+1;
      tmp.year = tm_brk.tm_year;
      tmp.lfd_nr = 0;
      write(fd,&tmp,sizeof(tmp));
    }
    else {
      /* Eintrag korrekt - auswerten und neuen schreiben */
      if ((tmp.day == tm_brk.tm_mday) && (tmp.mon == tm_brk.tm_mon+1) && (tmp.year == tm_brk.tm_year)) {
        /* Die nchste Mail an diesem Tag */
        tmp.lfd_nr++;
        if (tmp.lfd_nr > 1296) {
          /* Mehr als 1296 Mails an einem Tag verschickt - donnerwetter. Mehr geht beim besten
             Willen nicht */
          str[0] = '\0';
          close(fd);
          /* File unlocken */
          lockit(fd,F_UNLCK,0,SEEK_SET,0);
/*          box_error("NewBID:2) More than 1296 Mails/day send by THIS mailbox",-1);*/
          return;
        }
        write(fd,&tmp,sizeof(tmp));
      }
      else {
        /* Ein neuer Tag -> wieder bei 0 anfangen zu zhlen */
        tmp.day = tm_brk.tm_mday;
        tmp.mon = tm_brk.tm_mon+1;
        tmp.year = tm_brk.tm_year;
        tmp.lfd_nr = 0;
        write(fd,&tmp,sizeof(tmp));
      }
    }
  }
  
  /* Sooo - der aktuelle Eintrag steht jetzt in tmp */
  str[3] = Alpha[tmp.lfd_nr / 36];
  str[4] = Alpha[tmp.lfd_nr % 36];
  str[5] = '_';

  i = 0;
  while ((i <= 5) && (mycall[i] != '\0') && (mycall[i] != '-')) {
    str[i+6] = mycall[i];
    i++;
  }
  str[i+6] = '\0';
  
  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);

  close(fd);  
}




/* Gibt die maximale Lifetime dieses Boards zurck */
/*int get_lifetime(const char *board)
{
  char *tmp;
  int ok;
  s_ltlist *lt;
  int ltuse;
  
  tmp = (char *)strdup(board);
  Gross(tmp);
  
  ltuse = 30;
  ok = 0;
  lt = ltlist;
  while (ok==0) {
    if (lt == NULL) ok = 1;
    else {
      if (lt->board[0] == '*') ltuse = lt->lt;
      if (!strcmp(tmp,lt->board)) {
        ok = 1;
        ltuse = lt->lt;
      }
      lt = lt->next;
    }
  }

  free(tmp);
  return(ltuse);
}
*/





/* User-Informationen wenn vorhanden aus dem User-Info-File holen und
   in die Datenstruktur schreiben. */
int GetUserData(s_udata *udata)
{
  int f;
  char filename[500],line[500],str1[500],str2[5];
  int found=0,error,i,k;
  FILE *fs;
  
  
  strcpy(str1,udata->call);
  Klein(str1);
  COPY(str2,str1,0,3);
  sprintf(filename,"%s/userdata/%s.../%s",boxroot,str2,str1);
  
  
  if ((f = open(filename,O_RDONLY)) == -1) return(0);
  
  /* File locken */
  lockit(f,F_RDLCK,0,SEEK_SET,0);
  
  fs = fdopen(f,"r");
  
  while (fgets(line,499,fs) != NULL) {
    if (line[0] != '#') {	/* keine Kommentare */
      if ((i = POS('\n',line)) != -1) line[i] = '\0';
      if ((i = POS('\r',line)) != -1) line[i] = '\0';
    
      if ((line[0] != '\0') && ((i = POS(' ',line)) != -1)) {
        COPY(str1,line,0,i);
        COPY(line,line,i+1,strlen(line)-i-1);
        
        if (!strcmp(str1,"mybbs"))
           strcpy(udata->mybbs,line);
        if (!strcmp(str1,"name")) {
          if (strlen(line)>20) line[21] = '\0';
          strcpy(udata->name,line);
        }
        if (!strcmp(str1,"lastlogin"))
           udata->lastlogin = atoi(line);
        if (!strcmp(str1,"lastquit"))
           udata->lastquit = atoi(line);
        if (!strcmp(str1,"lastmybbs"))
           udata->lastmybbs = atoi(line);
        if (!strcmp(str1,"readlock"))
           udata->readlock = atoi(line);
        if (!strcmp(str1,"anzsend"))
           udata->anzsend = atoi(line);
        if (!strcmp(str1,"anzrec"))
           udata->anzrec = atoi(line);
        if (!strcmp(str1,"anzread"))
           udata->anzread = atoi(line);
        if (!strcmp(str1,"anzlogins"))
           udata->anzlogins = atoi(line);
        if (!strcmp(str1,"readlockfriends"))
           strcpy(udata->friends,line);
        if (!strcmp(str1,"password")) {
          KillSpacesLeft(line);
          KillSpacesRight(line);
          line[81] = '\0';
          strcpy(udata->passwd,line);
        }
        if (!strcmp(str1,"level"))
           udata->level = atoi(line);
      }
    }
  }

  /* File unlocken */
  lockit(f,F_UNLCK,0,SEEK_SET,0);
  
  
  close(f);
  
  
  return(1);
}




/* Holt die Daten des bergebenen Calls aus der Userdatenbank. Wird 1 zurckgegeben, ist der
 * User bekannt, bei 0 isser unbekannt.
 */
int get_mybbs(char *call, char *mybbs)
{
  s_udata udata;
  
  strcpy(udata.call,call);
  Klein(udata.call);
  
  if (GetUserData(&udata) == 1) {
    strcpy(mybbs,udata.mybbs);
    return(1);
  }
  
  /* Hier wird spter mal in der Datenbank der "externen" (noch nie eingeloggten) user
   * gesucht.
   */
  return(0);
}





/* Userdaten in das File schreiben */
void WriteUserData(const s_udata *udata)
{
  int f;
  char filename[500],line[256],str1[256],str2[5];
  
  strcpy(str1,udata->call);
  Klein(str1);
  COPY(str2,str1,0,3);
  sprintf(filename,"%s/userdata/%s...",boxroot,str2);
  
  /* Verzeichnis erzeugen, wenns noch nicht der Fall ist */
  if (mkdir(filename,448) == -1) {
  }
  sprintf(filename,"%s/%s",filename,str1);
  
  
  f = open(filename,(O_WRONLY|O_CREAT));
  
  /* Zugriffsrechte einstellen */
  fchmod(f,S_IRUSR|S_IWUSR);

  /* File zum schreiben locken */
  lockit(f,F_WRLCK,0,SEEK_SET,0);


  sprintf(str1,"mybbs %s\xA",udata->mybbs);
  write(f,str1,strlen(str1));
  sprintf(str1,"lastmybbs %i\xA",udata->lastmybbs);
  write(f,str1,strlen(str1));
  sprintf(str1,"name %s\xA",udata->name);
  write(f,str1,strlen(str1));
  sprintf(str1,"lastlogin %i\xA",udata->lastlogin);
  write(f,str1,strlen(str1));
  sprintf(str1,"lastquit %i\xA",udata->lastquit);
  write(f,str1,strlen(str1));
  sprintf(str1,"anzsend %i\xA",udata->anzsend);
  write(f,str1,strlen(str1));
  sprintf(str1,"anzrec %i\xA",udata->anzrec);
  write(f,str1,strlen(str1));
  sprintf(str1,"anzread %i\xA",udata->anzread);
  write(f,str1,strlen(str1));
  sprintf(str1,"anzlogins %i\xA",udata->anzlogins);
  write(f,str1,strlen(str1));
  sprintf(str1,"readlock %i\xA",udata->readlock);
  write(f,str1,strlen(str1));
  sprintf(str1,"readlockfriends %s\xA",udata->friends);
  write(f,str1,strlen(str1));
  sprintf(str1,"password %s\xA",udata->passwd);
  write(f,str1,strlen(str1));
  sprintf(str1,"level %i\xA",udata->level);
  write(f,str1,strlen(str1));
  
  /* Filemode einstellen - nur der Besitzer der Datei darf lesen, schreiben etc */
  fchmod(f,(S_IRUSR|S_IWUSR));
  
  /* File unlocken */
  lockit(f,F_UNLCK,0,SEEK_SET,0);
  
  close(f);
}



/* udata-Struktur leer machen */
void clear_udata(s_udata *udata)
{
  udata->call[0] = '\0';
  udata->mybbs[0] = '\0';
  udata->name[0] = '\0';
  udata->lastlogin = 0;
  udata->lastquit = 0;
  udata->lastmybbs = 0;
  udata->readlock = 0;
  udata->anzsend = 0;
  udata->anzrec = 0;
  udata->anzread = 0;
  udata->anzlogins = 0;
  udata->friends[0] = '\0';
  udata->flags = 0;
  udata->passwd[0] = '\0';
  udata->level = 0;

}


void _randomize()
{
  int t;
  
  t = (int)time(NULL);
  srand(t);
}
      
int _randint(short my_range)
{
  int i,j;
        
  i = RAND_MAX / my_range;
  i *= my_range;
  while ((j = rand()) >= i) continue;
  return (j % i) % my_range;
}

/* randomize erzeugt eine Zufallszahl in den Grenzen LO - HI */
/* wird fuer die Passwordberechnungen gebraucht              */

short dp_randomize(short low, short hiw)
{
  short erg;
  
  erg = _randint(hiw - low + 1);
  return (erg + low);
}



/* Einen Eintrag in die Checkliste machen */
void write_checklist(s_check *checkdata)
{
  int fd;
  char tmp[500];

  /* File ffnen / neu erstellen */  
  sprintf(tmp,"%s/system/check.dat",boxroot);
  fd = open(tmp,O_RDWR|O_CREAT);
  
  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(fd,F_WRLCK,0,SEEK_SET,0);
  
  /* Zugriffsrechte einstellen */
  fchmod(fd,S_IRUSR|S_IWUSR);
  
  /* Ans Ende der Datei springen */
  lseek(fd,0,SEEK_END);
  
  /* Den Eintrag ins File schreiben */
  write(fd,checkdata,sizeof(s_check));

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
  /* File schlieen */
  close(fd);
}



int getinfo_bid(char *bid, s_check *get_check)
{
  char tmp[500];
  int fd,count,found=0;
  s_check check;

  /* File ffnen / neu erstellen */  
  sprintf(tmp,"%s/system/check.dat",boxroot);
  if ((fd = open(tmp,O_RDONLY)) == -1) return(0);
  
  /* File zum lesen locken - Lesezugriffe sind noch mehr mglich, schreibzugriffe nicht mehr. */
  lockit(fd,F_RDLCK,0,SEEK_SET,0);

  while (!found && ((count = read(fd,&check,sizeof(s_check))) == sizeof(s_check))) {
    if (!strcmp(check.bid,bid)) {
      found = 1;
      memcpy(get_check,&check,sizeof(s_check));
    }
  }

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
  /* File schlieen */
  close(fd);
  
  if (found) return(1);
  return(0);
}



void delete_bid(char *bid)
{
  char tmp[500];
  int fd,count,found=0;
  s_check check;

  /* File ffnen / neu erstellen */  
  sprintf(tmp,"%s/system/check.dat",boxroot);
  if ((fd = open(tmp,O_RDWR)) == -1) return;

  /* File zum lesen locken - Lesezugriffe sind noch mehr mglich, schreibzugriffe nicht mehr. */
  lockit(fd,F_RDLCK,0,SEEK_SET,0);

  while (!found && ((count = read(fd,&check,sizeof(s_check))) == sizeof(s_check))) {
    if (!strcmp(check.bid,bid)) {
      check.flags |= MAIL_DEL;
      found = 1;
      lseek(fd,-sizeof(s_check),SEEK_CUR);
      write(fd,&check,sizeof(s_check));
    }
  }

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);

  /* File schlieen */
  close(fd);
}


/* Lscht ein File anhand ihrer Nummer in der Rubrik */
void delete_nr(char *rubrik, int nr)
{
  char tmp[500];
  int fd,found=FALSE;
  s_index index;
  off_t offs;
  
  /* Indexfile ffnen */
  sprintf(tmp,"%s/index/%s.idx",boxroot,rubrik);
  if ((fd = open(tmp,O_RDWR)) == -1) return;

  
  /* File zum lesen locken - Lesezugriffe sind noch mehr mglich, schreibzugriffe nicht mehr. */
  lockit(fd,F_RDLCK,0,SEEK_SET,0);

  offs = lseek(fd,(nr-1)*sizeof(s_index),SEEK_SET);
  
  if (offs == (nr-1)*sizeof(s_index)) {
    read(fd,&index,sizeof(s_index));
    lseek(fd,(nr-1)*sizeof(s_index),SEEK_SET);
    index.flags |= MAIL_DEL;
    write(fd,&index,sizeof(s_index));
  }

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);

  /* File schlieen */
  close(fd);
  
}


/* Einen Eintrag ins entsprechende Index-File machen 
 * Rckgabe: Die Nummer des Datensatzes, wo dieser Datensatz gespeichert wurde
 */
int write_index(s_index *index,char *rubrik)
{
  int fd;
  char tmp[500];
  off_t offset;
  
  /* File ffnen / neu erstellen */  
  sprintf(tmp,"%s/index/%s.idx",boxroot,rubrik);
  fd = open(tmp,O_RDWR|O_CREAT);
  
  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(fd,F_WRLCK,0,SEEK_SET,0);
  
  /* Zugriffsrechte einstellen */
  fchmod(fd,S_IRUSR|S_IWUSR);
  
  /* Ans Ende der Datei springen */
  offset = lseek(fd,0,SEEK_END);
  
  /* Den Eintrag ins File schreiben */
  write(fd,index,sizeof(s_index));
  
  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
  /* File schlieen */
  close(fd);
  
  return(offset/sizeof(s_index)+1);
}



int get_readlock(char *call, char *friends)
{
  s_udata udata;
  
  strcpy(udata.call,call);
  if (GetUserData(&udata) == 0) return(0);
  if (friends != NULL) strcpy(friends,udata.friends);
  return(udata.readlock);
}




/* Fr jeden eingeloggten User wird im ./userdata - Verzeichnis eine Datei erstellt, in der Infos
 * ber diese Station stehen. Hiermit wird diese Datei angelegt.
 */
void create_pidfile()
{
  int fd;
  char tmp[500];
  
  sprintf(tmp,"%s/userdata/%i.pid",boxroot,getpid());
  fd = open(tmp,O_CREAT|O_WRONLY);
  fchmod(fd,(S_IRUSR|S_IWUSR));
  close(fd);
}


/* Hier wird das pid-File fr diesen Prozess mit Daten gefllt. */
void fuelle_pidfile()
{
  int fd;
  char tmp[500];
  
  sprintf(tmp,"%s/userdata/%i.pid",boxroot,getpid());
  if ((fd = open(tmp,O_WRONLY)) == -1) return;
  
  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(fd,F_WRLCK,0,SEEK_SET,0);
  
  sprintf(tmp,"parent    %i\n",getppid());
  write(fd,tmp,strlen(tmp));
  sprintf(tmp,"call      %s\n",child_udata.call);
  write(fd,tmp,strlen(tmp));
  sprintf(tmp,"logintime %i\n",child_udata.lastlogin);
  write(fd,tmp,strlen(tmp));
  
  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
  close(fd);  
}



/* Gibt die Anzahl Mails aus, die das angegebene Rufzeichen im Index-File stehen hat. Ist das
 * Index-File nicht vorhanden, wird -1 zurckgegeben, sonst die Anzahl ungelschter Mails.
 */
int get_mailcount(char *call)
{
  char tmp[500];
  int fd,count,rdcnt=0;
  s_index index;
  
  sprintf(tmp,"%s/index/%s.idx",boxroot,call);
  if ((fd = open(tmp,O_RDONLY)) == -1) return(-1);
  
  /* File locken */
  lockit(fd,F_RDLCK,0,SEEK_SET,0);
  
  while ((count = read(fd,&index,sizeof(s_index))) == sizeof(s_index)) {
    if ((index.flags & MAIL_DEL) == 0) rdcnt++;
  }
  
  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
  close(fd);
  
  return(rdcnt);
}





/* Sucht nach Informationen ber das angegebene Board im boards.box - File (bzw. der verketteten Liste
 * dafr im Speicher.
 */
int get_boardinfo(char *board, int *lt, int *maxlt, char *newboard)
{
  s_boards *tmp;
  int found=FALSE;
  
  tmp = boardsroot;
  
  while ((tmp != NULL) && !found) {
    /* Gucken, ob dieser Eintrag unserer ist */
    if (!strcmp(tmp->rubrik,board)) {
      /* Jau, isses! */
      found = TRUE;
      *lt = tmp->lifetime;
      *maxlt = tmp->lifetime_max;
      strcpy(newboard,tmp->rubrik_after);
    }
    else tmp = tmp->next;
  }
  
  if (found) return(1);
  return(0);
}



/* Verschiebt eine Datei von source nach destination */
int move_file(char *source, char *destination)
{
  char s_fd,d_fd,count;
  char tmp[1000];
  
  if ((s_fd = open(source,O_RDWR)) == -1) return(-1);
  if ((d_fd = open(destination,O_WRONLY|O_CREAT)) == -1) {
    close(s_fd);
    return(-1);
  }

  /* Zugriffsrechte einstellen */
  fchmod(d_fd,S_IRUSR|S_IWUSR);

  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(s_fd,F_WRLCK,0,SEEK_SET,0);


  /* File kopieren in 1000 Byte-Blcken */
  while ((count = read(s_fd,tmp,1000)) > 0)
      write(d_fd,tmp,count);


  /* File unlocken */
  lockit(s_fd,F_UNLCK,0,SEEK_SET,0);

  close(s_fd);
  close(d_fd);

  unlink(source);

  return(0);
}





/* Diese Funktion erzeugt (wenn nicht vorhanden) eine Zeile "Read: " in der angegebenen Datei, und schreibt
 * da das bergebene Rufzeichen dahinter.
 */
void add_readline(char *filename, char *call)
{
  int fd,fd_tmp,fs;
  char tmpname[500];
  
  
  sprintf(tmpname,"/tmp/%i.vjboxd",getpid());

  if (move_file(filename,tmpname) == -1) return;
  

  unlink(tmpname);
}




/* Schreibt diese BID in der BID-Liste dazu */
void add_bidlist(char *bid)
{
  int fd,count;
  char tmp[500];
  unsigned long int writepos;

  sprintf(tmp,"%s/system/bid.dat",boxroot);
  if ((fd = open(tmp,O_RDWR|O_CREAT)) == -1) return;

  /* Zugriffsrechte einstellen */
  fchmod(fd,S_IRUSR|S_IWUSR);

  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(fd,F_WRLCK,0,SEEK_SET,0);


  count = read(fd,&writepos,sizeof(writepos));

  if (count != sizeof(writepos)) {
    /* File ist leer */
    writepos = 0;
  }
  if (writepos > 65000) writepos = 0;

  lseek(fd,sizeof(writepos)+(12*writepos),SEEK_SET);
  write(fd,bid,12);

  lseek(fd,0,SEEK_SET);
  writepos++;
  write(fd,&writepos,sizeof(writepos));

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);

  close(fd);
}



/* Guckt, ob die bergebene BID im BID-Pool vorhanden ist.
 * Rckgabe: TRUE wenn ja, sonst FALSE
 */
int bid_checkforbid(char *bid)
{
  int fd,count;
  char tmp[500];
  int found=FALSE;
  char readbid[13];

  sprintf(tmp,"%s/system/bid.dat",boxroot);
  if ((fd = open(tmp,O_RDONLY)) == -1) return(FALSE);

  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(fd,F_RDLCK,0,SEEK_SET,0);

  read(fd,tmp,4);

  while (((count = read(fd,readbid,12)) == 12) && !found)
  {
    readbid[12] = '\0';
    if (!strcmp(bid,readbid))
       found = TRUE;
  }

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);

  close(fd);
  
  return(found);
}



/* Gibt 1 zurck, wenn die bergebene Rubrik lokal als Userfile angesehen wird, sonst 0 */
int is_userfile(char *rubrik)
{
  return(rufz(rubrik));
}




/* Gibt 1 zurck, wenn die Mail gelesen werden darf, sonst 0 */
int readpermission(int flags,char *rubrik,int rl)
{
  if (rl == 0) return(1);

  /* Die Mail ist eine ffentliche -> Anzeigen */
  if ((flags & MAIL_USER) == 0) return(1);
  
  /* Wir sind als SysOp eingeloggt. */
  if ((child_udata.flags & UDATA_SYSOP) == UDATA_SYSOP) return(1);
  
  /* Uns gehrt die Rubrik */
  if (!strcmp(rubrik,child_udata.call)) return(1);
  
  /* Die Mail wurde schon gelesen und rl steht auf 1 */
  if ((rl == 1) && ((flags & MAIL_READ) == MAIL_READ)) return(1);
  
  return(0);
}



/* Die aktuelle Verbindung wird getrennt - eine Zeile ins Logfile schreiben */
void log_me()
{
  int fd;
  char tmp[500];
  time_t now;
  struct tm l_now;
  struct tms proctime;

  
  now = times(&proctime);
  now = time(NULL);
  l_now = *gmtime(&now);
  
  sprintf(tmp,"%s/log/%s%i.log",boxroot,months_small[l_now.tm_mon],l_now.tm_year);
  if ((fd = open(tmp,O_CREAT|O_WRONLY|O_APPEND)) == 0) return;

  /* Zugriffsrechte einstellen */
  fchmod(fd,S_IRUSR|S_IWUSR);
  
  /* Jetzt wird die Zeile, die gespeichert werden soll, erzeugt. */
  sprintf(tmp,"%6s %10d %10d %6.2f\n"
             ,child_udata.call,child_udata.lastlogin,now,proctime.tms_utime/(double)clktck);
  
  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(fd,F_WRLCK,0,SEEK_SET,0);

  write(fd,tmp,strlen(tmp));
  
  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
  close(fd);
}




void GetTimeDiff(struct tm_diff *dest, int diff)
{
  /* Sekunden */
  dest->secs = diff % 60;
  diff = diff / 60;
  
  /* Minuten */
  dest->mins = diff % 60;
  diff = diff / 60;
  
  /* Hours */
  dest->hours = diff % 24;
  diff = diff / 24;
  
  /* Days */
  dest->days = diff;
}



void inc_readcount(char *bid, char *rubrik)
{
  char tmp[500];
  int fd,count,found=FALSE;
  s_check check;
  s_index index;

  /* File ffnen */
  sprintf(tmp,"%s/system/check.dat",boxroot);
  if ((fd = open(tmp,O_RDWR)) == -1) return;
  
  /* File locken */
  lockit(fd,F_WRLCK,0,SEEK_SET,0);

  while (!found && ((count = read(fd,&check,sizeof(s_check))) == sizeof(s_check))) {
    if (!strcmp(check.bid,bid)) {
      found = 1;
      lseek(fd,-sizeof(s_check),SEEK_CUR);
      check.read++;
      write(fd,&check,sizeof(s_check));
    }
  }

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
  /* File schlieen */
  close(fd);
  
  
  /************************* INDEX-File **********************************/
  if (rubrik[0] != '\0') {
    sprintf(tmp,"%s/index/%s.idx",boxroot,rubrik);
    if ((fd = open(tmp,O_RDWR)) == -1) return;
    
    /* File lesen */
    lockit(fd,F_WRLCK,0,SEEK_SET,0);
  
    found = FALSE;
    while (!found && ((count = read(fd,&index,sizeof(s_index))) == sizeof(s_index))) {
      if (!strcmp(index.bid,bid)) {
        found = 1;
        lseek(fd,-sizeof(s_index),SEEK_CUR);
        index.read++;
        write(fd,&index,sizeof(s_index));
      }
    }
    
    /* File unlocken */
    lockit(fd,F_UNLCK,0,SEEK_SET,0);
  
    /* File schlieen */
    close(fd);
  }
}



void add_readlist(char *filename, char *call)
{
  int fd,feddich=FALSE,i,len;
  char tmp[500],tmp2[500];
  FILE *fs;
  char findcall[50];
  long pos;


  sprintf(findcall, " %s ", call);
  
  if ((fd = open(filename,O_RDWR)) == -1) return;
  
  /* File zum Schreiben locken  */
  lockit(fd,F_WRLCK,0,SEEK_SET,0);

  /* Den zustndigen Stream 'rausfinden */
  if ((fs = fdopen(fd,"rw")) == NULL) {
    close(fd);
    return;
  }

  pos = ftell(fs);
  while ((fgets(tmp,500,fs)) != NULL && (!feddich)) {
    if (tmp[0] == '\0')
      feddich = TRUE;
    else {
      if ((i = POS('\n', tmp)) != -1) tmp[i] = '\0';
      len = strlen(tmp);
      if ((i = POS(' ',tmp)) != -1) {
        COPY(tmp2,tmp,0,i);
        COPY(tmp,tmp,i,strlen(tmp)-i);
        if (!strcmp(tmp2,"Read:")) {
          if (strstr(tmp, findcall) == NULL)
          {
            KillSpacesRight( tmp );
            KillSpacesLeft( tmp );
            if (tmp[0] == '\0')
              sprintf(tmp2, "Read: %s", call);
            else
              sprintf( tmp2, "Read: %s %s", tmp, call);
            strcat(tmp2, addChars(' ',len-strlen(tmp2)));
            lseek(fd, pos, SEEK_SET);
            write( fd, tmp2, strlen(tmp2));
          }
          feddich = TRUE;
        }
      }
    }
    pos = ftell(fs);
  }

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);

  /* File schlieen */
  close(fd);
  
}


int user_read_mail( char *filename, char *call )
{
  int fd,feddich=FALSE,i,len;
  char tmp[500],tmp2[500];
  FILE *fs;
  char findcall[50];
  int rueckgabe=FALSE;


  sprintf(findcall, " %s ", call);
  
  if ((fd = open(filename,O_RDONLY)) == -1) return;

  /* File zum lesen locken - Lesezugriffe sind noch mehr mglich, schreibzugriffe nicht mehr. */
  lockit(fd,F_RDLCK,0,SEEK_SET,0);

  /* Den zustndigen Stream 'rausfinden */
  if ((fs = fdopen(fd,"r")) == NULL) {
    lockit(fd,F_UNLCK,0,SEEK_SET,0);
    close(fd);
    return;
  }

  while ((fgets(tmp,500,fs)) != NULL && (!feddich)) {
    if (tmp[0] == '\0')
      feddich = TRUE;
    else {
      if ((i = POS('\n', tmp)) != -1) tmp[i] = '\0';
      len = strlen(tmp);
      if ((i = POS(' ',tmp)) != -1) {
        COPY(tmp2,tmp,0,i);
        COPY(tmp,tmp,i,strlen(tmp)-i);
        if (!strcmp(tmp2,"Read:")) {
          if (strstr(tmp, findcall) != NULL)
            rueckgabe = TRUE;
          feddich = TRUE;
        }
      }
    }
  }

  /* File unlocken */
  lockit(fd,F_UNLCK,0,SEEK_SET,0);

  /* File schlieen */
  close(fd);

  return rueckgabe;
}


/* Wieviele Mailserver sind online? */
int get_mailservercount()
{
  s_server *tmp;
  int count=0;
  
  tmp = serverroot;
  
  while (tmp != NULL) {
    count++;
    tmp = tmp->next;
  }
  
  return(count);
}



/* Guckt, ob der aktuelle User S&F machen darf (mittels cookie oder passwort). */
int is_sf_user()
{
  char tmp[500];
  int ok;
  
  /* OK, die Station ist eine S&F-Station */
  if ((child_udata.flags & UDATA_SF) == UDATA_SF) return(1);
  
  if (child_udata.level != 8) return(0);
  
  /* Passwort eingetragen und eingeloggt: S&F mglich */
  if ((child_udata.passwd[0] != '\0') && logged_in)
  {
    child_udata.flags |= UDATA_SF;
    return(1);
  }
  
  /* Wenn jemand Forwarden darf, mu die Datei "forward.<pid>" in der vjbox-root
   * existieren. Die PID ist die PID unseres vjboxd-Prozesses.
   */
  sprintf(tmp,"%s/forward.%i",boxroot,getpid());
  
  ok = file_exist(tmp);
  
  if (ok)
  {
    child_udata.flags |= UDATA_SF;
    return(1);
  }
  else return(0);
}



/* Sendet diese (neue) BID in Richtung Router */
void router_sendbid_fifo(char *bid)
{
  uchar len;

  if (bidfifofd == -1) return;

  /* File zum schreiben locken - Lesezugriffe sind nicht mehr mglich */
  lockit(bidfifofd,F_WRLCK,0,SEEK_SET,0);

  len = strlen(bid);
  write(bidfifofd,&len,sizeof(len));
  write(bidfifofd,&bid,len);

  /* File unlocken */
  lockit(bidfifofd,F_UNLCK,0,SEEK_SET,0);

}

